// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"fmt"
	"strings"
	"time"
)

func (r *ReleaseAssistant) generateChangelog(changes changes, releases releases) string {
	text := "# Changelog\n\n"

	for _, release := range releases {
		text += r.releaseToChangelogText(changes, *release)
	}

	return text
}

func (r *ReleaseAssistant) releaseToChangelogText(changes changes, release release) string {
	text := fmt.Sprintf("## %s%s ", r.Config.VersionPrefix, release.Version.String())

	if release.Name != "" {
		text += fmt.Sprintf("(%s)", release.Name)
	}

	releaseTime := time.Unix(release.Timestamp, 0)

	text += fmt.Sprintf(
		" - %d.%d.%d\n\n",
		releaseTime.Day(),
		releaseTime.Month(),
		releaseTime.Year(),
	)

	if release.Description != "" {
		text += stringToMdComment(release.Description)
	}

	containedChanges := changes.filterIncludedInRelease(&release)
	if len(containedChanges) == 0 {
		return text + "No Changes \n\n"
	}

	if containedChanges.containsBreakingChanges() {
		text += "THIS VERSION CONTAINS BREAKING CHANGES! READ CAREFULLY.\n"
	}

	for _, changeType := range r.Config.ChangeTypes {
		changesOfChangeType := containedChanges.filterByType(changeType.ID)

		if len(changesOfChangeType) == 0 {
			continue
		}

		text += fmt.Sprintf("### %s\n\n", changeType.Name)

		changesOfChangeType = changesOfChangeType.sortAlphabetically()

		for _, change := range changesOfChangeType {
			text += r.changeToChangelogText(change)
		}
	}

	return text + "\n\n"
}

func (r *ReleaseAssistant) changeToChangelogText(change *change) string {
	text := fmt.Sprintf("- %s \n", change.Title)

	if change.Description != "" {
		text += "	" + stringToMdComment(change.Description)
	}

	if change.Breaking {
		breakingText := "THIS CHANGE IS BREAKING!"

		if change.BreakingMigrationGuide != "" {
			breakingText += " The following steps are required to migrate to this version: \n" + change.BreakingMigrationGuide
		}

		text += "	" + stringToMdComment(breakingText)
	}

	return text
}

func stringToMdComment(str string) string {
	return fmt.Sprintf("> %s\n\n", strings.ReplaceAll(str, "\n", " <br>"))
}
