// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"fmt"
	"io/fs"
	"io/ioutil"
	"os"
	"path/filepath"
	"strings"

	"golang.org/x/exp/slices"
	"gopkg.in/yaml.v3"
)

type changeType struct {
	ID          int16  `yaml:"id"`
	Name        string `yaml:"name"`
	Description string `yaml:"description"`
}

type change struct {
	ID                     string `yaml:"id"`
	Title                  string `yaml:"title"`
	Type                   int16  `yaml:"type"`
	Description            string `yaml:"description"`
	Breaking               bool   `yaml:"breaking"`
	BreakingMigrationGuide string `yaml:"breaking_migration_guide"`
	Timestamp              int64  `yaml:"timestamp"`
}

type changes []*change

func (c *change) getPrettyName(inputchangeTypes changeTypes) string {
	foundChangeType := inputchangeTypes.getByID(c.Type)
	if foundChangeType == nil {
		// Fallback in case a change type gets deleted
		foundChangeType = &changeType{}
	}

	name := fmt.Sprintf("[%s] %s", strings.ToUpper(foundChangeType.Name), c.Title)

	return name
}

func (c *changes) getIDsAsList() *[]string {
	ids := make([]string, 0, len(*c))

	for _, change := range *c {
		ids = append(ids, change.ID)
	}

	return &ids
}

func (c *changes) filterIncludedInRelease(release *release) changes {
	changes := make(changes, 0, len(*c))

	for _, change := range *c {
		if slices.ContainsFunc[string](release.Changes, func(s string) bool {
			return s == change.ID
		}) {
			changes = append(changes, change)
		}
	}

	return changes
}

func (c *changes) filterByType(typeID int16) changes {
	changes := make(changes, 0, len(*c))

	for _, change := range *c {
		if change.Type == typeID {
			changes = append(changes, change)
		}
	}

	return changes
}

func (c *changes) containsBreakingChanges() bool {
	for _, change := range *c {
		if change.Breaking {
			return true
		}
	}

	return false
}

func (c *changes) sortAlphabetically() changes {
	changes := *c

	slices.SortFunc(changes, func(i, j *change) bool {
		return i.Title < j.Title
	})

	return changes
}

func (c *changes) getPrettyNamesAsList(changeTypes changeTypes) *[]string {
	ids := make([]string, 0, len(*c))

	for _, change := range *c {
		ids = append(ids, change.getPrettyName(changeTypes))
	}

	return &ids
}

func (c changeTypes) getNamesAsList() []string {
	names := make([]string, 0, len(c))

	for _, changeType := range c {
		names = append(names, changeType.Name)
	}

	return names
}

func (c changeTypes) getIDByName(name string) int16 {
	for _, changeType := range c {
		if changeType.Name == name {
			return changeType.ID
		}
	}

	return -1
}

func (c changeTypes) getByID(id int16) *changeType {
	for _, changeType := range c {
		if changeType.ID == id {
			return &changeType
		}
	}

	return nil
}

// GetFileName returns the filename this change should have.
func (c *change) GetFileName() string {
	return fmt.Sprintf("%d_%s.yaml", c.Timestamp, c.ID)
}

func (r *ReleaseAssistant) saveChange(change *change) error {
	changeData, _ := yaml.Marshal(change)

	return ioutil.WriteFile(r.Config.ChangeDirectory+"/"+change.GetFileName(), changeData, fs.ModePerm)
}

func (r *ReleaseAssistant) loadChange(file string) (*change, error) {
	file = filepath.Clean(file)

	changeData, err := ioutil.ReadFile(file)
	if err != nil {
		return nil, err
	}

	change := &change{}
	err = yaml.Unmarshal(changeData, change)

	if err != nil {
		return nil, err
	}

	return change, nil
}

func (r *ReleaseAssistant) loadAllChanges() (changes, error) {
	entries, err := os.ReadDir(r.Config.ChangeDirectory)
	if err != nil {
		return nil, err
	}

	changes := make([]*change, 0, len(entries))

	for _, entry := range entries {
		if entry.IsDir() {
			continue
		}

		change, err := r.loadChange(r.Config.ChangeDirectory + "/" + entry.Name())
		if err != nil {
			return nil, err
		}

		changes = append(changes, change)
	}

	return changes, nil
}

func (r *ReleaseAssistant) promptChange(change *change) error {
	var err error

	change.Type, err = r.askForChangeType()
	if err != nil {
		return err
	}

	change.Title, err = r.askForSingleLine("title", false)
	if err != nil {
		return err
	}

	change.Description, err = r.askForMultiline("description")
	if err != nil {
		return err
	}

	change.Breaking, err = r.askForIsBreaking()
	if err != nil {
		return err
	}

	change.BreakingMigrationGuide = ""

	if change.Breaking {
		change.BreakingMigrationGuide, err = r.askForMultiline("migration-guide")
		if err != nil {
			return err
		}
	}

	return nil
}
