// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"io/fs"
	"io/ioutil"
	"os"

	"github.com/fatih/color"
	"gopkg.in/yaml.v3"
)

var (
	configFile    = "./releasecli.yaml"
	defaultConfig = config{
		ConfigVersion:    1,
		ChangeDirectory:  "changes",
		ReleaseDirectory: "releases",
		ChangeTypes: []changeType{
			{
				ID:          1,
				Name:        "Added",
				Description: "Added or extended a feature",
			},
			{
				ID:          2,
				Name:        "Changed",
				Description: "An existing feature changed",
			},
			{
				ID:          3,
				Name:        "Removed",
				Description: "An existing feature was removed or dismantled",
			},
			{
				ID:          4,
				Name:        "Deprecated",
				Description: "An existing feature was deprecated or marked for removal",
			},
			{
				ID:          5,
				Name:        "Fixed",
				Description: "Fixed a bug",
			},
			{
				ID:          6,
				Name:        "Documented",
				Description: "Extended or corrected the documentation",
			},
		},
		ChangelogFile: "CHANGELOG.md",
		VersionPrefix: "v",
	}
)

type changeTypes []changeType

type config struct {
	ConfigVersion    int16       `yaml:"config_version"`
	ChangeDirectory  string      `yaml:"change_directory"`
	ReleaseDirectory string      `yaml:"release_directory"`
	ChangeTypes      changeTypes `yaml:"change_types"`
	ChangelogFile    string      `yaml:"changelog_file"`
	VersionPrefix    string      `yaml:"version_prefix"`
}

func ensureThatConfigExists() (bool, error) {
	exists, err := pathExists(configFile)
	if err != nil {
		return false, err
	}

	if exists {
		return true, nil
	}

	c := color.New(color.FgRed)

	_, err = c.Println("Project not initialized. Please run 'releasecli init' first.")
	if err != nil {
		return false, err
	}

	return false, nil
}

func initializeProject() error {
	configData, err := yaml.Marshal(defaultConfig)
	if err != nil {
		return err
	}

	err = ioutil.WriteFile(configFile, configData, fs.ModePerm)
	if err != nil {
		return err
	}

	c := color.New(color.FgHiGreen)

	_, err = c.Println("Project has been initialized successfully.")
	if err != nil {
		return err
	}

	return nil
}

func readConfig() (*config, error) {
	if exists, err := ensureThatConfigExists(); err != nil || !exists {
		if err != nil {
			return nil, err
		}

		os.Exit(1)
	}

	configData, err := ioutil.ReadFile(configFile)
	if err != nil {
		return nil, err
	}

	config := &config{}
	err = yaml.Unmarshal(configData, config)

	return config, err
}
