// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"github.com/AlecAivazis/survey/v2"
	"golang.org/x/text/cases"
	"golang.org/x/text/language"
)

func (r *ReleaseAssistant) askForChangeType() (int16, error) {
	response := ""
	prompt := &survey.Select{
		Message: "What kind of change do you want to create?",
		Options: r.Config.ChangeTypes.getNamesAsList(),
		Description: func(_ string, index int) string {
			return r.Config.ChangeTypes[index].Description
		},
	}

	if err := survey.AskOne(prompt, &response); err != nil {
		return 0, err
	}

	changeType := r.Config.ChangeTypes.getIDByName(response)

	return changeType, nil
}

func (r *ReleaseAssistant) askForSingleLine(subject string, requireConfirmation bool) (string, error) {
	response := ""

	if requireConfirmation {
		addText := false
		confirmPrompt := &survey.Confirm{
			Message: "Do you want to add a " + subject + "?",
			Default: false,
		}

		err := survey.AskOne(confirmPrompt, &addText)
		if err != nil || !addText {
			return "", err
		}
	}

	prompt := &survey.Input{
		Message: cases.Title(language.English).String(subject) + ":",
		Default: "",
	}
	if err := survey.AskOne(prompt, &response, survey.WithValidator(survey.MinLength(1))); err != nil {
		return "", err
	}

	return response, nil
}

func (r *ReleaseAssistant) askForMultiline(subject string) (string, error) {
	text := ""

	addText := false
	confirmPrompt := &survey.Confirm{
		Message: "Do you want to add a " + subject + "?",
		Default: false,
	}

	err := survey.AskOne(confirmPrompt, &addText)
	if err != nil {
		return "", err
	}

	if addText {
		descriptionPrompt := &survey.Multiline{
			Message: cases.Title(language.English).String(subject) + ":",
		}

		err := survey.AskOne(descriptionPrompt, &text)
		if err != nil {
			return "", err
		}
	}

	return text, nil
}

func (r *ReleaseAssistant) askForIsBreaking() (bool, error) {
	breaking := false

	confirmPrompt := &survey.Confirm{
		Message: "Is this change breaking?",
		Default: false,
	}
	if err := survey.AskOne(confirmPrompt, &breaking); err != nil {
		return false, err
	}

	return breaking, nil
}

func (r *ReleaseAssistant) askForVersionBump(version *version) error {
	versionBumpIndex := 0

	patchBump := *version
	patchBump.Patch++

	minorBump := *version
	minorBump.Minor++
	minorBump.Patch = 0

	majorBump := *version
	majorBump.Major++
	majorBump.Patch = 0
	majorBump.Minor = 0

	prompt := &survey.Select{
		Message: "What kind of version bump should happen with this release:",
		Options: []string{"Patch", "Minor", "Major"},
		Description: func(_ string, index int) string {
			switch index {
			case 0:
				return r.Config.VersionPrefix + patchBump.String()
			case 1:
				return r.Config.VersionPrefix + minorBump.String()
			case 2:
				return r.Config.VersionPrefix + majorBump.String()
			default:
				return ""
			}
		},
	}
	if err := survey.AskOne(prompt, &versionBumpIndex); err != nil {
		return err
	}

	switch versionBumpIndex {
	case 0:
		version.Patch++
	case 1:
		version.Minor++
		version.Patch = 0
	case 2:
		version.Major++
		version.Patch = 0
		version.Minor = 0
	}

	return nil
}

func (r *ReleaseAssistant) askSelectChanges(availableChanges changes) (changes, error) {
	selectedChanges := make(changes, 0, len(availableChanges))

	if len(availableChanges) == 0 {
		return selectedChanges, nil
	}

	selectedIndexes := []int{}
	prompt := &survey.MultiSelect{
		Message: "Which changes should be included?",
		Options: *availableChanges.getPrettyNamesAsList(r.Config.ChangeTypes),
		Default: true,
	}

	if err := survey.AskOne(prompt, &selectedIndexes); err != nil {
		return nil, err
	}

	for _, selectedIndex := range selectedIndexes {
		change := *availableChanges[selectedIndex]
		selectedChanges = append(selectedChanges, &change)
	}

	return selectedChanges, nil
}
