// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// Package src contains all code relevant for running the command line tool.
package src

import (
	"fmt"
	"io/fs"
	"io/ioutil"
	"sort"
	"time"

	"github.com/google/uuid"
	"golang.org/x/exp/slices"
)

// ReleaseAssistant is the main ReleaseAssistant type.
type ReleaseAssistant struct {
	Config *config
}

// NewReleaseAssistant returns a new NewReleaseAssistant instance.
func NewReleaseAssistant() *ReleaseAssistant {
	return &ReleaseAssistant{
		Config: nil,
	}
}

func (r *ReleaseAssistant) init() error {
	config, err := readConfig()
	if err != nil {
		return err
	}

	r.Config = config

	if err := r.ensureDirectoryExists(r.Config.ChangeDirectory); err != nil {
		return err
	}

	return r.ensureDirectoryExists(r.Config.ReleaseDirectory)
}

// RunInitializeProject runs all required steps to initialize the project.
func (r *ReleaseAssistant) RunInitializeProject() error {
	return initializeProject()
}

// RunCreateChange runs all required steps to create a change.
func (r *ReleaseAssistant) RunCreateChange() error {
	if err := r.init(); err != nil {
		return err
	}

	change := &change{
		ID:        uuid.NewString(),
		Timestamp: time.Now().Unix(),
	}

	if err := r.promptChange(change); err != nil {
		return err
	}

	return r.saveChange(change)
}

// RunCreateRelease runs all required steps to create a release.
func (r *ReleaseAssistant) RunCreateRelease() error {
	if err := r.init(); err != nil {
		return err
	}

	allChanges, err := r.loadAllChanges()
	if err != nil {
		return err
	}

	releases, err := r.loadAllReleases()
	if err != nil {
		return err
	}

	releasedChangeIDs := releases.getAllChangeIDs()

	var unreleasedChanges changes = filter[*change](allChanges, func(c *change) bool {
		return !slices.ContainsFunc[string](*releasedChangeIDs, func(s string) bool {
			return s == c.ID
		})
	})

	latestRelease := releases.getLatestRelease()

	release := release{
		ID:        uuid.NewString(),
		Timestamp: time.Now().Unix(),
		Version:   latestRelease.Version,
	}

	if err := r.promptRelease(&release, unreleasedChanges); err != nil {
		return err
	}

	return r.saveRelease(&release)
}

// RunGenerateChangelog runs all required steps to generate the changelog.
func (r *ReleaseAssistant) RunGenerateChangelog(stdout bool, latest bool) error {
	if err := r.init(); err != nil {
		return err
	}

	changes, err := r.loadAllChanges()
	if err != nil {
		return err
	}

	releases, err := r.loadAllReleases()
	if err != nil {
		return err
	}

	if latest {
		latestRelease := releases.getLatestRelease()

		releases = make([]*release, 0, 1)
		releases = append(releases, latestRelease)
	}

	sort.Sort(byVersion(releases))

	changelogText := r.generateChangelog(changes, releases)

	if stdout {
		fmt.Print(changelogText)

		return nil
	}

	return ioutil.WriteFile(r.Config.ChangelogFile, []byte(changelogText), fs.ModePerm)
}

func filter[T any](ss []T, test func(T) bool) []T {
	ret := make([]T, 0, len(ss))

	for _, s := range ss {
		if test(s) {
			ret = append(ret, s)
		}
	}

	return ret
}
