// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"fmt"
	"io/fs"
	"io/ioutil"
	"os"
	"path/filepath"

	"gopkg.in/yaml.v3"
)

type version struct {
	Major int64 `yaml:"major"`
	Minor int64 `yaml:"minor"`
	Patch int64 `yaml:"patch"`
}

type release struct {
	ID          string   `yaml:"id"`
	Name        string   `yaml:"name"`
	Description string   `yaml:"description"`
	Version     version  `yaml:"version"`
	Changes     []string `yaml:"changes"`
	Timestamp   int64    `yaml:"timestamp"`
}

type releases []*release

type byVersion releases

func (s byVersion) Swap(i, j int) {
	s[i], s[j] = s[j], s[i]
}

func (s byVersion) Less(i, j int) bool {
	return s[i].Version.Major > s[j].Version.Major ||
		s[i].Version.Minor > s[j].Version.Minor ||
		s[i].Version.Patch > s[j].Version.Patch
}

func (s byVersion) Len() int {
	return len(s)
}

func (v version) String() string {
	return fmt.Sprintf("%d.%d.%d", v.Major, v.Minor, v.Patch)
}

func (r *releases) getAllChangeIDs() *[]string {
	changes := []string{}

	for _, release := range *r {
		changes = append(changes, release.Changes...)
	}

	return &changes
}

func (r *releases) getLatestRelease() *release {
	latestRelease := &release{}

	for _, release := range *r {
		if release.Timestamp > latestRelease.Timestamp {
			latestRelease = release
		}
	}

	return latestRelease
}

func (r *release) getFileName() string {
	return fmt.Sprintf("%d_%s.yaml", r.Timestamp, r.ID)
}

func (r *ReleaseAssistant) saveRelease(change *release) error {
	changeData, _ := yaml.Marshal(change)

	return ioutil.WriteFile(r.Config.ReleaseDirectory+"/"+change.getFileName(), changeData, fs.ModePerm)
}

func (r *ReleaseAssistant) loadRelease(file string) (*release, error) {
	file = filepath.Clean(file)

	releaseData, err := ioutil.ReadFile(file)
	if err != nil {
		return nil, err
	}

	release := &release{}
	err = yaml.Unmarshal(releaseData, release)

	if err != nil {
		return nil, err
	}

	return release, nil
}

func (r *ReleaseAssistant) loadAllReleases() (releases, error) {
	entries, err := os.ReadDir(r.Config.ReleaseDirectory)
	if err != nil {
		return nil, err
	}

	releases := make([]*release, 0, len(entries))

	for _, entry := range entries {
		if entry.IsDir() {
			continue
		}

		release, err := r.loadRelease(r.Config.ReleaseDirectory + "/" + entry.Name())
		if err != nil {
			return nil, err
		}

		releases = append(releases, release)
	}

	return releases, nil
}

func (r *ReleaseAssistant) promptRelease(release *release, changes changes) error {
	var err error

	release.Name, err = r.askForSingleLine("name", true)
	if err != nil {
		return err
	}

	release.Description, err = r.askForMultiline("description")
	if err != nil {
		return err
	}

	selectedChanges, err := r.askSelectChanges(changes)
	if err != nil {
		return err
	}

	release.Changes = *selectedChanges.getIDsAsList()

	return r.askForVersionBump(&release.Version)
}
